package br.ufsc.cne.codemaster;

import com.google.gson.Gson;

import br.ufsc.cne.codemaster.control.Control;
import br.ufsc.cne.codemaster.objetos.Professor;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.UUID;
import javax.servlet.ServletException;
import javax.servlet.annotation.MultipartConfig;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.Part;

import org.apache.log4j.Logger;

@WebServlet("/FileUploadServlet")
@MultipartConfig(fileSizeThreshold = 1024 * 10, // 10 KB 
        maxFileSize = 1024 * 1024 * 20, // 20 MB
        maxRequestSize = 1024 * 1024 * 200)   	// 200 MB
public class Upload extends HttpServlet {

    private final String UPLOAD_DIR = "/var/lib/tomcat8/webapps/ROOT/projects/files";
    private Control control;
    static Logger log = Logger.getLogger(Upload.class);

    protected void doPost(HttpServletRequest request, HttpServletResponse response) {
        try {
            ProjectSettings settings = this.getProjectSettings(request);
            
            // Log4j - configuração básica
//            BasicConfigurator.configure();

            //cria controle
            control = new Control();

            //INICIO salvar turma no banco
            Integer idTurma = control.salvarTurma(request.getParameter("nomeTurma"), ((Professor) request.getSession().getAttribute("professor")).getId());
            //FIM salvar turma
            log.info("Id da turma criada" + idTurma);
            //Inicio salvar evento no bd
            Integer idEvento = control.salvarEvento("evento" + idTurma, idTurma);
            //fim salvar evento
            log.info("Id da turma criada" + idEvento);
            
            //Analisar todos os projetos
            HashMap<String, String> allJSONGrades = this.sendAllProjectsAndGetAllGrades(request, settings, idEvento);

            int amount = allJSONGrades.size();
            int counter = 0;
            String param;
            String msg;
            for (HashMap.Entry<String, String> entry : allJSONGrades.entrySet()) {
                param = "param" + counter;
                msg = entry.getValue();
                request.setAttribute(param, msg);
                counter++;
            }
            request.setAttribute("amount", amount);
            if (settings.getProjectType().equals("Snap!")) {
                getServletContext().getRequestDispatcher("/professor_resultado_snap.jsp").forward(request, response);
            } else if (settings.getProjectType().equals("AppInventor")) {
                getServletContext().getRequestDispatcher("/professor_resultado_appinventor.jsp").forward(request, response);
            }
        } catch (Exception e) {
            log.error("Não foi possível enviar todos os projetos para avaliação.", e);
            try {
                getServletContext().getRequestDispatcher("/erro.jsp").forward(request, response);
            } catch (Exception ex) {
                log.warn("Não foi possível redirecionar para a página de erro.", ex);
            }
        }
    }

    private String getFileExtension(String fileName) {
        String extension = "";
        int i = fileName.lastIndexOf('.');
        if (i > 0) {
            extension = fileName.substring(i + 1);
        }
        return extension;
    }

    private String getFileName(Part part) {
        String contentDisp = part.getHeader("content-disposition");
        String[] tokens = contentDisp.split(";");
        for (String token : tokens) {
            if (token.trim().startsWith("filename")) {
                return token.substring(token.indexOf("=") + 2, token.length() - 1);
            }
        }
        return "";
    }

    private ProjectSettings getProjectSettings(HttpServletRequest request) {
        ProjectSettings settings = new ProjectSettings();
        settings.setProjectType(request.getParameter("language"));
        settings.setTelasIsRelevant(request.getParameter("checkTelas") != null);
        settings.setEventosIsRelevant(request.getParameter("checkEventos") != null);
        settings.setInterfaceUsuarioIsRelevant(request.getParameter("checkInterface") != null);
        settings.setNomeacaoProcedimentosVariaveisIsRelevant(request.getParameter("checkNomeacao") != null);
        settings.setAbstracaoProcedimentosIsRelevant(request.getParameter("checkAbstracaoProcedimentos") != null);
        settings.setLacosIsRelevant(request.getParameter("checkLacos") != null);
        settings.setCondicionaisIsRelevant(request.getParameter("checkCondicionais") != null);
        settings.setListasIsRelevant(request.getParameter("checkListas") != null);
        settings.setPersistenciaDadosIsRelevant(request.getParameter("checkPersistencia") != null);
        settings.setSensoresIsRelevant(request.getParameter("checkSensores") != null);
        settings.setMidiaIsRelevant(request.getParameter("checkMidia") != null);
        settings.setSocialIsRelevant(request.getParameter("checkSocial") != null);
        settings.setConectividadeIsRelevant(request.getParameter("checkConectividade") != null);
        settings.setDesenhoAnimacaoIsRelevant(request.getParameter("checkDesenhoAnimacao") != null);
        settings.setAppOperadoresIsRelevant(request.getParameter("checkOperadoresApp") != null);
        settings.setLogicIsRelevant(request.getParameter("checkLogica") != null);
        settings.setParallelismIsRelevant(request.getParameter("checkParalelismo") != null);
        settings.setSynchronizationIsRelevant(request.getParameter("checkSincronizacao") != null);
        settings.setUserInteractivityIsRelevant(request.getParameter("checkInteratividadeUsuario") != null);
        settings.setAbstractionIsRelevant(request.getParameter("checkAbstracao") != null);
        settings.setDataRepresentationIsRelevant(request.getParameter("checkRepresentacaoDados") != null);
        settings.setFlowControlIsRelevant(request.getParameter("checkControleFluxo") != null);
        settings.setSnapOperatorsIsRelevant(request.getParameter("checkOperadoresSnap") != null);
        return settings;
    }

    private HashMap<String, String> sendAllProjectsAndGetAllGrades(HttpServletRequest request, ProjectSettings settings, Integer idEvento) throws IOException, ServletException {
        HashMap<String, String> allJSONGrades = new HashMap<>();
        for (Part part : request.getParts()) {
            if (part.getName().equals("file")) {
                String fileName = getFileName(part);
                settings.setProjectName(fileName);
                UUID uniqueID = UUID.randomUUID();
                String filePath = UPLOAD_DIR + File.separator + String.valueOf(uniqueID) + fileName;
                File saved = this.saveFile(part.getInputStream(), filePath);
                HTTPClient client = new HTTPClient();
                String JSONGrade = client.sendProjectToServerAndGetResponse(saved, settings);
                allJSONGrades.put(fileName, JSONGrade);
                //salvar no banco
                saveProjectBD(fileName, idEvento, saved, settings, JSONGrade);
            }
        }
        return allJSONGrades;
    }

    private void saveProjectBD(String fileName, Integer idEvento, File projeto, ProjectSettings settings, String grade) {
        //INICIO salvar aluno(grupo) e projeto no banco
        String nomeDoGrupo = fileName.substring(0, fileName.length() - 4);
        Integer idGrupoAlunos = control.salvarGrupoDeAlunos(nomeDoGrupo, idEvento);
        //salvar projeto
        Gson gsonSettings = new Gson();
        String stringSettings = gsonSettings.toJson(settings);
        Integer idProjeto = control.salvarProjetoBd(fileName, settings.getProjectType(), grade, stringSettings, idGrupoAlunos);
        //FIM salvar aluno(grupo) e projeto
        
        //salva arquivo de projeto com id de projeto
        File projetoId = new File(UPLOAD_DIR + File.separator + "id_" + idProjeto + "." + getFileExtension(fileName));
        saveFileId(projeto, projetoId);
        projeto.delete();

    }
    
    private void saveFileId(File original, File copia) {
        try {
            FileInputStream in = new FileInputStream(original);
            FileOutputStream out = new FileOutputStream(copia);
            byte[] buf = new byte[1024];
            int len;
            while ((len = in.read(buf)) > 0) {
                out.write(buf, 0, len);
            }
            out.flush();
            out.close();
            in.close();
        } catch (IOException e) {
            log.error("Não foi possível salvar o arquivo do projeto.", e);
        }
    }

    private File saveFile(InputStream uploadedInputStream, String serverLocation) {
        createUploadDirIfNotExists();
        try {
            OutputStream outpuStream = new FileOutputStream(new File(serverLocation));
            int read = 0;
            byte[] bytes = new byte[1024];
            outpuStream = new FileOutputStream(new File(serverLocation));
            while ((read = uploadedInputStream.read(bytes)) != -1) {
                outpuStream.write(bytes, 0, read);
            }
            outpuStream.flush();
            outpuStream.close();
        } catch (IOException e) {
            log.error("Não foi possível salvar o arquivo do projeto.", e);
        }
        return new File(serverLocation);
    }

    private void createUploadDirIfNotExists() {
        File fileSaveDir = new File(UPLOAD_DIR);
        if (!fileSaveDir.exists()) {
            fileSaveDir.mkdirs();
        }
    }
}
