package br.ufsc.cne.codemaster;

import com.google.gson.Gson;

import br.ufsc.cne.codemaster.control.Control;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.UUID;
import javax.servlet.annotation.MultipartConfig;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.Part;
import org.apache.log4j.Logger;


@WebServlet("/FileUploadServletAluno")
@MultipartConfig(fileSizeThreshold = 1024 * 10, // 10 KB 
        maxFileSize = 1024 * 1024 * 20, // 20 MB
        maxRequestSize = 1024 * 1024 * 200)   	// 200 MB
public class UploadAluno extends HttpServlet {

    private final String UPLOAD_DIR = "/var/lib/tomcat8/webapps/ROOT/projects/files";
    static Logger log = Logger.getLogger(UploadAluno.class);

    protected void doPost(HttpServletRequest request, HttpServletResponse response) {
        //BasicConfigurator.configure();
        try {
            Part filePart = request.getPart("file");
            UUID uniqueID = UUID.randomUUID();
            String fileName = this.getFileName(filePart);
            String ext = this.getFileExtension(fileName);
            String filePath = UPLOAD_DIR + File.separator + String.valueOf(uniqueID) + fileName;
            File saved = saveFile(filePart.getInputStream(), filePath);
            ProjectSettings settings = null;
            String grade = null;
            if (ext.equalsIgnoreCase("xml")) {
                settings = new ProjectSettings();
                settings.setProjectType("Snap!");
                settings.setProjectName(fileName);
                grade = this.sendProjectAndGetGrade(saved, settings);
                request.setAttribute("msg", grade);
                getServletContext().getRequestDispatcher("/aluno_resultado_snap.jsp").forward(request, response);
            } else if (ext.equalsIgnoreCase("aia")) {
                settings = new ProjectSettings();
                settings.setProjectType("AppInventor");
                settings.setProjectName(fileName);
                grade = this.sendProjectAndGetGrade(saved, settings);
                request.setAttribute("msg", grade);
                getServletContext().getRequestDispatcher("/aluno_resultado_appinventor.jsp").forward(request, response);
            }

            //salvar projeto no banco
            Control control = new Control();
            Gson gsonSettings = new Gson();
            String stringSettings = gsonSettings.toJson(settings);
            Integer idTurma = null;
            Integer idProjeto = control.salvarProjetoBd(fileName, settings.getProjectType(), grade, stringSettings, idTurma);
            
            //salva arquivo de projeto com id de projeto
            File projetoId = new File(UPLOAD_DIR + File.separator + "id_" + idProjeto + "." + ext);
            saveFileId(saved, projetoId);
            saved.delete();
        } catch (Exception e) {
            log.error("Não foi possível enviar o projeto para avaliação.", e);
            try {
                getServletContext().getRequestDispatcher("/erro.jsp").forward(request, response);
            } catch (Exception ex) {
                log.warn("Não foi possível redirecionar para a página de erro.", ex);
            }
        }
    }


    private String getFileExtension(String fileName) {
        String extension = "";
        int i = fileName.lastIndexOf('.');
        if (i > 0) {
            extension = fileName.substring(i + 1);
        }
        return extension;
    }

    
    private String getFileName(Part part) {
        String contentDisp = part.getHeader("content-disposition");
        System.out.println("content-disposition header= " + contentDisp);
        String[] tokens = contentDisp.split(";");
        for (String token : tokens) {
            if (token.trim().startsWith("filename")) {
                return token.substring(token.indexOf("=") + 2, token.length() - 1);
            }
        }
        return "";
    }

    
    private String sendProjectAndGetGrade(File projectFile, ProjectSettings settings) {
        String jsonGrade = "";
        HTTPClient client = new HTTPClient();
        jsonGrade = client.sendProjectToServerAndGetResponse(projectFile, settings);
        return jsonGrade;
    }
    
    

    private File saveFile(InputStream uploadedInputStream, String serverLocation) {
        createUploadDirIfNotExists();
	try {
            OutputStream outpuStream = new FileOutputStream(new File(serverLocation));
            int read = 0;
            byte[] bytes = new byte[1024];
            outpuStream = new FileOutputStream(new File(serverLocation));
            while ((read = uploadedInputStream.read(bytes)) != -1) {
                outpuStream.write(bytes, 0, read);
            }
            outpuStream.flush();
            outpuStream.close();
	} catch (IOException e) {
            log.error("Não foi possível salvar o arquivo do projeto.", e);
        }
        return new File(serverLocation);
    }
    
    private void saveFileId(File original, File copia) {
        try {
            FileInputStream in = new FileInputStream(original);
            FileOutputStream out = new FileOutputStream(copia);
            byte[] buf = new byte[1024];
            int len;
            while ((len = in.read(buf)) > 0) {
                out.write(buf, 0, len);
            }
            out.flush();
            out.close();
            in.close();
        } catch (IOException e) {
            log.error("Não foi possível salvar o arquivo do projeto.", e);
        }
    }

    private void createUploadDirIfNotExists() {
        File fileSaveDir = new File(UPLOAD_DIR);
        if (!fileSaveDir.exists()) {
            fileSaveDir.mkdirs();
        }
    }
}
