/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package br.ufsc.cne.codemaster.control;

import java.io.UnsupportedEncodingException;
import java.security.GeneralSecurityException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.List;
import java.util.Locale;
import java.util.ResourceBundle;

import org.apache.log4j.Logger;

import br.ufsc.cne.codemaster.mail.Mail;
import br.ufsc.cne.codemaster.objetos.Evento;
import br.ufsc.cne.codemaster.objetos.GrupoAlunos;
import br.ufsc.cne.codemaster.objetos.Professor;
import br.ufsc.cne.codemaster.objetos.Projeto;
import br.ufsc.cne.codemaster.objetos.Turma;
import br.ufsc.cne.codemaster.util.*;

/**
 *
 * @author Matheus Demetrio <matheus.fdemetrio@gmail.com>
 */
public class Control {
    
    static Logger log = Logger.getLogger(Control.class);
    
    public Control() {
    }
    
    public String gerarHash(String msg) throws NoSuchAlgorithmException, UnsupportedEncodingException {
        MessageDigest algorithm = MessageDigest.getInstance("SHA-256");
        byte messageDigest[] = algorithm.digest(msg.getBytes("UTF-8"));

        StringBuilder hexString = new StringBuilder();
        for (byte b : messageDigest) {
            hexString.append(String.format("%02X", 0xFF & b));
        }
        String msghex = hexString.toString();

        return msghex;
    }

    public Integer salvarTurma(String nomeTurma, Integer idProfessor) {
        Turma turma = new Turma();
        turma.setNome(nomeTurma);
        turma.setIdProfessor(idProfessor);

        TurmaDAO turmaDAO = new TurmaDAO();
        turmaDAO.salvar(turma);

        return turma.getId();
    }

    public Integer salvarEvento(String nomeEvento, Integer idTurma) {
        Evento evento = new Evento();
        evento.setNome(nomeEvento);
        evento.setIdTurma(idTurma);
        
        java.util.Date data = new java.util.Date(System.currentTimeMillis());  
        evento.setData(data);

        EventoDAO eventoDAO = new EventoDAO();
        eventoDAO.salvar(evento);

        return evento.getId();
    }

    public Integer salvarGrupoDeAlunos(String nomeGrupo, Integer idEvento) {
        GrupoAlunos ga = new GrupoAlunos();
        ga.setNome(nomeGrupo);
        ga.setIdEvento(idEvento);

        GrupoAlunosDAO grupoAlunosDAO = new GrupoAlunosDAO();
        grupoAlunosDAO.salvar(ga);

        return ga.getId();
    }

    public Integer salvarProjetoBd(String nomeArquivo, String tipoProjeto, String resultadoAvaliacao, String configAvaliacao, Integer idGrupoAlunos) {
        Projeto projeto = new Projeto();
        projeto.setNomeArquivo(nomeArquivo);
        projeto.setTipoProjeto(tipoProjeto);
        projeto.setResultadoAvaliacao(resultadoAvaliacao);
        projeto.setConfigAvaliacao(configAvaliacao);
        projeto.setIdGrupoAlunos(idGrupoAlunos);

        ProjetoDAO projetoDAO = new ProjetoDAO();
        if (idGrupoAlunos == null) {
            projetoDAO.salvarAlunoAnonimo(projeto);
        } else {
            projetoDAO.salvar(projeto);
        }

        return projeto.getId();
    }
    
    public boolean atualizarSenhaProfessor (String email, String language, String country){
        boolean resp = false;
        try {
            int i = (int) (10000000 + Math.random() * 99999999);
            
            String senha = String.valueOf(i);
            String hashSenha = gerarHash(senha);
            
            ProfessorDAO professorDAO = new ProfessorDAO();
            resp = professorDAO.salvarSenha(email, hashSenha);
            
            Locale locale = new Locale(language, country);
            ResourceBundle messages = ResourceBundle.getBundle("EmailBundle", locale);
            
            if (resp){
                Mail mail = new Mail();
                mail.EnviarEmail(email, messages.getString("AssuntoEmailRecSenha"), messages.getString("MensagemEmailRecSenha") + " " + senha + "\n\n\n" + messages.getString("MensagemAutomatica"));
            }
            
            return resp;
        } catch (NoSuchAlgorithmException ex) {
            log.error(ex);
        } catch (UnsupportedEncodingException | GeneralSecurityException ex) {
            log.error(ex);
        }
        return resp;
    }

    public Integer salvarProfessor(String nome, String email, String senha, String cidade, String disciplina, String instituicao, String pais, String uf, String language, String country) {
        try {
            Professor professor = new Professor();
            
            String hashSenha = gerarHash(senha);
            String hashEmail = gerarHash(email);

            professor.setNome(nome);
            professor.setEmail(email);
            professor.setSenha(hashSenha);
            professor.setCidade(cidade);
            professor.setDisciplina(disciplina);
            professor.setInstituicao(instituicao);
            professor.setPais(pais);
            professor.setUf(uf);
            professor.setHashEmail(hashEmail);
            professor.setEmailValido(false);

            ProfessorDAO professorDAO = new ProfessorDAO();
            professorDAO.salvar(professor);
            
            Mail mail = new Mail();
            
            Locale locale = new Locale(language, country);
            ResourceBundle messages = ResourceBundle.getBundle("EmailBundle", locale);
            
            try {
                mail.EnviarEmail(email, messages.getString("AssuntoEmailValidacao"), messages.getString("ConteudoEmailValidacao") + " http://apps.computacaonaescola.ufsc.br:8080/Front/professor_login.jsp?val=" + hashEmail + "\n\n\n" + messages.getString("MensagemAutomatica"));
            } catch (GeneralSecurityException ex) {
                log.error(ex);
            }

            return professor.getId();

        } catch (NoSuchAlgorithmException | UnsupportedEncodingException ex) {
            log.error(ex);
        }
        return null;
    }

    public Boolean existeProfessor(String email) {
        ProfessorDAO professorDAO = new ProfessorDAO();
        return professorDAO.login(email, null) != null;
    }

    public List<Turma> listarTurmas(Integer id_professor) {
        TurmaDAO turmaDAO = new TurmaDAO();
        return turmaDAO.buscaTurmas(id_professor);
    }

    public List<Projeto> listarProjetos(Integer id_turma) {
        ProjetoDAO projetoDAO = new ProjetoDAO();
        return projetoDAO.buscaProjetos(id_turma);
    }

    public Professor loginProfessor(String email, String senha) {
        try {
            ProfessorDAO professorDAO = new ProfessorDAO();
           
            String hashSenha = gerarHash(senha);

            return professorDAO.login(email, hashSenha);
        } catch (NoSuchAlgorithmException | UnsupportedEncodingException ex) {
            log.error(ex);
        }
        return null;
    }
    
    public boolean validarEmail(String hash){
        ProfessorDAO professorDAO = new ProfessorDAO();
        return professorDAO.validarEmail(hash);
    }

}
