package br.ufsc.cne.codemaster.restgrader.appinventorgrader;

import java.io.File;
import org.apache.log4j.Logger;

import org.json.simple.JSONObject;
import org.w3c.dom.Document;

import br.ufsc.cne.codemaster.restgrader.appinventorgrader.codigo.Codigo;
import br.ufsc.cne.codemaster.restgrader.appinventorgrader.conceitos.AbstracaoProcedimentos;
import br.ufsc.cne.codemaster.restgrader.appinventorgrader.conceitos.ConceitoCT;
import br.ufsc.cne.codemaster.restgrader.appinventorgrader.conceitos.Condicionais;
import br.ufsc.cne.codemaster.restgrader.appinventorgrader.conceitos.Conectividade;
import br.ufsc.cne.codemaster.restgrader.appinventorgrader.conceitos.DesenhoAnimacao;
import br.ufsc.cne.codemaster.restgrader.appinventorgrader.conceitos.Eventos;
import br.ufsc.cne.codemaster.restgrader.appinventorgrader.conceitos.InterfaceUsuario;
import br.ufsc.cne.codemaster.restgrader.appinventorgrader.conceitos.Lacos;
import br.ufsc.cne.codemaster.restgrader.appinventorgrader.conceitos.Listas;
import br.ufsc.cne.codemaster.restgrader.appinventorgrader.conceitos.Midia;
import br.ufsc.cne.codemaster.restgrader.appinventorgrader.conceitos.NomeacaoProcedimentosVariaveis;
import br.ufsc.cne.codemaster.restgrader.appinventorgrader.conceitos.Operadores;
import br.ufsc.cne.codemaster.restgrader.appinventorgrader.conceitos.PersistenciaDados;
import br.ufsc.cne.codemaster.restgrader.appinventorgrader.conceitos.Sensores;
import br.ufsc.cne.codemaster.restgrader.appinventorgrader.conceitos.Social;
import br.ufsc.cne.codemaster.restgrader.appinventorgrader.conceitos.Telas;
import br.ufsc.cne.codemaster.restgrader.appinventorgrader.io.Descompactador;
import br.ufsc.cne.codemaster.restgrader.appinventorgrader.parse.JSONParse;
import br.ufsc.cne.codemaster.restgrader.appinventorgrader.parse.XMLParse;
import br.ufsc.cne.codemaster.restgrader.computacaonaescola.restgrader.Grade;
import br.ufsc.cne.codemaster.restgrader.computacaonaescola.restgrader.Grader;
import br.ufsc.cne.codemaster.restgrader.computacaonaescola.restgrader.ProjectSettings;


/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */



/**
 *
 * @author Matheus Demetrio <matheus.fdemetrio@gmail.com>
 */
public class AppInventorGrader extends Grader{

    static Logger log = Logger.getLogger(AppInventorGrader.class);
    
    @Override
    public Grade gradeProject(File project, String projectName, ProjectSettings settings) {
        
        log.info("Iniciando a análise e avaliação de código do App Inventor! PROJETO: " + projectName);
        
        File projetoAIA = project;

        String destinoUnzip = "webapps" + File.separator + "RESTGrader" + File.separator + "projetos-aia-extraidos" + File.separator + "projeto";

        XMLParse parserXML = new XMLParse();
        JSONParse parserJSON = new JSONParse();
        Codigo codigo = new Codigo();
        AppInventorGrade grade = new AppInventorGrade();
        grade.setNomeProjeto(projectName);
        Integer total = 0;

        //para cada tela do App é gerado um arquivo xml e um json, cada arquivo é carregado e enviado para a instância codigo:
        Descompactador descomp = new Descompactador();
        descomp.unZip(projetoAIA, destinoUnzip);

        for (String caminho : descomp.listaJson) {
            parserJSON.carregarJSON(caminho);
            JSONObject jsonObject = parserJSON.carregarJSON(caminho);
            if(jsonObject != null) {
            	codigo.addCodigoVisual(jsonObject);
            }
        }
        for (String caminho : descomp.listaBky) {
            parserXML.carregarXML(caminho);
            Document documento = parserXML.carregarXML(caminho);
            if(documento != null) {
            	codigo.addCodigoLogica(documento);
            }
			
        }
        
        codigo.gerarEstatisticasLogica();
        codigo.gerarEstatisticasVisual();
        
        int qtdConceitosAnalisados = 0;

        
        //Analisando Telas
        if(settings.getTelasIsRelevant()) {
            ConceitoCT telas = new Telas(codigo); telas.avaliaCodigo(); grade.setTelas(telas.getNota());
            qtdConceitosAnalisados++;
            total += grade.getTelas();
            log.info("Sua nota em Telas foi: " + telas.getNota() + " | Feedback: " + telas.getFeedback());
        }else {
        	grade.setTelas(0);
        }

        //Interface Usuario
        if(settings.getInterfaceUsuarioIsRelevant()) {
            ConceitoCT iu = new InterfaceUsuario(codigo); iu.avaliaCodigo(); grade.setInterfaceUsuario(iu.getNota());
            qtdConceitosAnalisados++;
            total += grade.getInterfaceUsuario();
            log.info("Sua nota em Interface Usuário foi: " + iu.getNota() + " | Feedback: " + iu.getFeedback());
        }else {
        	grade.setInterfaceUsuario(0);
        }

        //Nomeacao procedimentos e variaveis
        if(settings.getNomeacaoProcedimentosVariaveisIsRelevant()) {
        	ConceitoCT nome = new NomeacaoProcedimentosVariaveis(codigo); nome.avaliaCodigo(); grade.setNomeacaoProcedimentosVariaveis(nome.getNota());
        	qtdConceitosAnalisados++;
        	total += grade.getNomeacaoProcedimentosVariaveis();
            log.info("Sua nota em Nomeação de Procedimentos e Variáveis foi: " + nome.getNota() + " | Feedback: " + nome.getFeedback());
        }else {
        	grade.setNomeacaoProcedimentosVariaveis(0);
        }
        
        
        //Eventos
        if(settings.getEventosIsRelevant()) {
            ConceitoCT ev = new Eventos(codigo); ev.avaliaCodigo(); grade.setEventos(ev.getNota());
            qtdConceitosAnalisados++;
            total += grade.getEventos();
            log.info("Sua nota em Eventos foi: " + ev.getNota() + " | Feedback: " + ev.getFeedback());
        }else {
        	grade.setEventos(0);
        }

        
        //Procedimentos
        if(settings.getAbstracaoProcedimentosIsRelevant()) {
            ConceitoCT proc = new AbstracaoProcedimentos(codigo); proc.avaliaCodigo(); grade.setAbstracaoProcedimentos(proc.getNota());
            qtdConceitosAnalisados++;
            total += grade.getAbstracaoProcedimentos();
            log.info("Sua nota em Abstração de Procedimentos foi: " + proc.getNota() + " | Feedback: " + proc.getFeedback());
        }else {
        	grade.setAbstracaoProcedimentos(0);
        }

        //Laços
        if(settings.getLacosIsRelevant()) {
            ConceitoCT lacos = new Lacos(codigo);lacos.avaliaCodigo();grade.setLacos(lacos.getNota());
            qtdConceitosAnalisados++;
            total += grade.getLacos();
            log.info("Sua nota em Laços foi: " + lacos.getNota() + " | Feedback: " + lacos.getFeedback());
        }else {
        	grade.setLacos(0);
        }

        //Condicionais
        if(settings.getCondicionaisIsRelevant()) {
            ConceitoCT condicionais = new Condicionais(codigo);condicionais.avaliaCodigo();grade.setCondicionais(condicionais.getNota());
            qtdConceitosAnalisados++;
            total += grade.getCondicionais();
            log.info("Sua nota em Condicionais foi: " + condicionais.getNota() + " | Feedback: " + condicionais.getFeedback());
        }else {
        	grade.setCondicionais(0);
        }
        
        //Listas
        if(settings.getListasIsRelevant()) {
            ConceitoCT listas = new Listas(codigo); listas.avaliaCodigo(); grade.setListas(listas.getNota());
            qtdConceitosAnalisados++;
            total += grade.getListas();
            log.info("Sua nota em Listas foi: " + listas.getNota() + " | Feedback: " + listas.getFeedback());
        }else {
        	grade.setListas(0);
        }

        //Persistencia de Dados
        if(settings.getPersistenciaDadosIsRelevant()) {
            ConceitoCT dados = new PersistenciaDados(codigo); dados.avaliaCodigo(); grade.setPersistenciaDados(dados.getNota());
            qtdConceitosAnalisados++;
            total += grade.getPersistenciaDados();
            log.info("Sua nota em Persistencia de Dados foi: " + dados.getNota() + " | Feedback: " + dados.getFeedback());
        }else {
        	grade.setPersistenciaDados(0);
        }

        //Sensores
        if(settings.getSensoresIsRelevant()) {
            ConceitoCT sensores = new Sensores(codigo); sensores.avaliaCodigo(); grade.setSensores(sensores.getNota());
            qtdConceitosAnalisados++;
            total += grade.getSensores();
            log.info("Sua nota em Sensores foi: " + sensores.getNota() + " | Feedback: " + sensores.getFeedback());
        }else {
        	grade.setSensores(0);
        }
        
        //Midia
        if(settings.getMidiaIsRelevant()) {
            ConceitoCT midia = new Midia(codigo); midia.avaliaCodigo(); grade.setMidia(midia.getNota());
            qtdConceitosAnalisados++;
            total += grade.getMidia();
            log.info("Sua nota em Midia foi: " + midia.getNota() + " | Feedback: " + midia.getFeedback());
        }else {
        	grade.setMidia(0);
        }

        //Social
        if(settings.getSocialIsRelevant()) {
            ConceitoCT social = new Social(codigo); social.avaliaCodigo(); grade.setSocial(social.getNota());
            qtdConceitosAnalisados++;
            total += grade.getSocial();
            log.info("Sua nota em Social foi: " + social.getNota() + " | Feedback: " + social.getFeedback());
        }else {
        	grade.setSocial(0);
        }

        //Conectividade
        if(settings.getConectividadeIsRelevant()) {
            ConceitoCT conectividade = new Conectividade(codigo); conectividade.avaliaCodigo(); grade.setConectividade(conectividade.getNota());
            qtdConceitosAnalisados++;
            total += grade.getConectividade();
            log.info("Sua nota em Conectividade foi: " + conectividade.getNota() + " | Feedback: " + conectividade.getFeedback());
        }else {
        	grade.setConectividade(0);
        }

        //Desenho Animaçao
        if(settings.getDesenhoAnimacaoIsRelevant()) {
            ConceitoCT desenhoAnima = new DesenhoAnimacao(codigo); desenhoAnima.avaliaCodigo(); grade.setDesenhoAnimacao(desenhoAnima.getNota());
            qtdConceitosAnalisados++;
            total += grade.getDesenhoAnimacao();
            log.info("Sua nota em Desenho e Animação foi: " + desenhoAnima.getNota() + " | Feedback: " + desenhoAnima.getFeedback());
        }else {
        	grade.setDesenhoAnimacao(0);
        }
        
        //Operadores 
        if(settings.getAppOperadoresIsRelevant()) {
            ConceitoCT operadores = new Operadores(codigo); operadores.avaliaCodigo(); grade.setOperadores(operadores.getNota());
            qtdConceitosAnalisados++;
            total += grade.getOperadores();
            log.info("Sua nota em Operadores foi: " + operadores.getNota());
        }else {
            grade.setOperadores(0);
        }

        
        //coloca total em grade
        grade.setTotal(total);
        
        //calcula nota final
        if(qtdConceitosAnalisados == 0) {
        	qtdConceitosAnalisados = 1;
        }
        double notaFinal = (double)total / (((double)qtdConceitosAnalisados*3)/10);
        grade.setNota(notaFinal);
        
        //define nível ninja
        String nivelProjeto = "";
        if(notaFinal < 1) {
        	nivelProjeto = "faixa branca";
        } else if(notaFinal < 2) {
        	nivelProjeto = "faixa amarela";
        } else if(notaFinal < 3) {
        	nivelProjeto = "faixa laranja";
        } else if(notaFinal < 4) {
        	nivelProjeto = "faixa vermelha";
        } else if(notaFinal < 5) {
        	nivelProjeto = "faixa roxa";
        } else if(notaFinal < 6) {
        	nivelProjeto = "faixa azul";
        } else if(notaFinal < 7) {
        	nivelProjeto = "faixa turquesa";
        } else if(notaFinal < 8) {
        	nivelProjeto = "faixa verde";
        }else if(notaFinal < 9) {
        	nivelProjeto = "faixa marrom";
        }else if(notaFinal <= 10) {
        	nivelProjeto = "faixa preta";
        }
        grade.setNivel(nivelProjeto);

        log.info("Análise e avaliação de código App Inventor finalizada - retornando resultado!");
        
        return grade;
    }

}
