package br.ufsc.cne.codemaster.restgrader.computacaonaescola.restgrader;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import javax.ws.rs.Consumes;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import com.google.gson.Gson;
import com.sun.jersey.core.header.FormDataContentDisposition;
import com.sun.jersey.multipart.FormDataParam;

import br.ufsc.cne.codemaster.restgrader.appinventorgrader.AppInventorGrade;
import br.ufsc.cne.codemaster.restgrader.appinventorgrader.AppInventorGrader;
import br.ufsc.cne.codemaster.restgrader.snapgrader.SnapGrade;
import br.ufsc.cne.codemaster.restgrader.snapgrader.SnapMaster;

import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.UUID;
import org.apache.log4j.Logger;

@Path("/files")
public class RESTGrader {

    private final String UPLOAD_DIR = "/var/lib/tomcat8/webapps/RESTGrader/projects/files";
    private final String TOKEN = "IAUAHJ7686JHDKL86765KJJGGHH1267IOLMDF";
    static Logger log = Logger.getLogger(RESTGrader.class);

    @POST
    @Path("/upload")
    @Consumes(MediaType.MULTIPART_FORM_DATA)
    public Response uploadFile(@FormDataParam("file") InputStream fileInputStream,
            @FormDataParam("file") FormDataContentDisposition contentDispositionHeader,
            @FormDataParam("settings") String jsonSettings) {
        try {
            Gson gson = new Gson();
            ProjectSettings settings = gson.fromJson(jsonSettings, ProjectSettings.class);
            String fileName = settings.getProjectName();
            UUID uniqueID = UUID.randomUUID();
            String filePath = UPLOAD_DIR + File.separator + String.valueOf(uniqueID) + fileName;
            File project = saveFile(fileInputStream, filePath);
            Grader grader = null;
            Grade grade = null;
            
            log.info("Mensagem recebida - começando processo de análise e avaliação!");

            if (settings.getProjectType().equals("Snap!") && settings.getTOKEN().equals(TOKEN)) {
                grader = new SnapMaster();
                grade = new SnapGrade();
            } else if (settings.getProjectType().equals("AppInventor") && settings.getTOKEN().equals(TOKEN)) {
                grader = new AppInventorGrader();
                grade = new AppInventorGrade();
            } else {
                log.error("Arquivo recebido inválido!");
                return Response.status(403).entity("Erro: Arquivo inválido.").build();
            }

            grade = grader.gradeProject(project, fileName, settings);
            String jsonGrade = gson.toJson(grade);
            try {
                Files.delete(Paths.get(filePath));
            } catch (Exception e) {
                log.warn("Problema ao excluir arquivos de projeto do servidor.");
            }
            
            log.info("Processo de análise e avaliação concluído - retornando resultado!");
            return Response.status(200).entity(jsonGrade).build();
        } catch (Exception ex) {
            ex.printStackTrace();
            return Response.status(200).entity("").build();
        }
    }

    private String getFileExtension(String filename) {
        return filename.substring(filename.lastIndexOf(".") + 1);
    }

    private File saveFile(InputStream uploadedInputStream, String serverLocation) {
        createUploadDirIfNotExists();
        try {
            OutputStream outpuStream = new FileOutputStream(new File(serverLocation));
            int read = 0;
            byte[] bytes = new byte[1024];
            outpuStream = new FileOutputStream(new File(serverLocation));
            while ((read = uploadedInputStream.read(bytes)) != -1) {
                outpuStream.write(bytes, 0, read);
            }
            outpuStream.flush();
            outpuStream.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return new File(serverLocation);
    }

    private void createUploadDirIfNotExists() {
        File fileSaveDir = new File(UPLOAD_DIR);
        if (!fileSaveDir.exists()) {
            fileSaveDir.mkdirs();
        }
    }
}
