package br.ufsc.cne.codemaster.restgrader.snapgrader;
import java.util.ArrayList;
import java.util.Map;

import br.ufsc.cne.codemaster.restgrader.computacaonaescola.restgrader.ProjectSettings;


public class SnapGrader {

    public SnapGrade assessProject(Map<String, Integer> tokens, String projectName, ProjectSettings settings) {
        SnapGrade grade = new SnapGrade();
        grade.setProjectName(projectName);
        if(settings.getLogicIsRelevant())
            grade.setLogic(this.assessLogic(tokens));
        if(settings.getParallelismIsRelevant())
            grade.setParallelism(this.assessParallelism(tokens));
        if(settings.getSynchronizationIsRelevant())
            grade.setSynchronization(this.assesSynchronization(tokens));
        if(settings.getUserInteractivityIsRelevant())
            grade.setUserInteractivity(this.assessUserInteractivity(tokens));
        if(settings.getAbstractionIsRelevant())
            grade.setAbstraction(this.assessAbstraction(tokens));
        if(settings.getDataRepresentationIsRelevant())
            grade.setDataRepresentation(this.assessDataRepresentation(tokens));
        if(settings.getFlowControlIsRelevant())
            grade.setFlowControl(this.assessFlowControl(tokens));
        if(settings.getSnapOperatorsIsRelevant())
            grade.setOperators(this.assessOperators(tokens));
        grade.setTotal(this.assessTotal(grade));
        grade.setScore(this.asessScore(grade, settings));
        grade.setProjectLevel(this.assessProjectLevel(grade));
        return grade;
    }


///////////////////////////////////////////////////////////////////////////////    
/////////////////////////////  Assessment  ////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
    
    private Integer assessLogic(Map<String, Integer> tokens) {
        Boolean doIf = tokens.containsKey("doIf");
        Boolean doIfElse = tokens.containsKey("doIfElse");
        Boolean reportNot = tokens.containsKey("reportNot");
        Boolean reportAnd = tokens.containsKey("reportAnd");
        Boolean reportOr = tokens.containsKey("reportOr");
        if(reportNot || reportAnd || reportOr) {
            return 3;
        } 
        if(doIfElse) {
            return 2;
        }
        if(doIf) {
            return 1;
        }
        return 0;
    }
    
    private Integer assessParallelism(Map<String, Integer> tokens) {
        Integer greenFlagAmount = tokens.get("receiveGo") == null ? 0 : tokens.get("receiveGo");
        Integer spritePressedAmount = tokens.get("receiveInteraction") == null ? 0 : tokens.get("receiveInteraction");
        Integer receiveMsgOrCloneAmount = tokens.get("receiveOnClone") == null ? 0 : tokens.get("receiveOnClone");
        receiveMsgOrCloneAmount += tokens.get("receiveMessage") == null ? 0 : tokens.get("receiveMessage");
        Integer sensingAmount = tokens.get("reportKeyPressed") == null ? 0 : tokens.get("reportKeyPressed");
        sensingAmount += tokens.get("reportTouchingObject") == null ? 0 : tokens.get("reportTouchingObject");
        sensingAmount += tokens.get("reportTouchingColor") == null ? 0 : tokens.get("reportTouchingColor");
        sensingAmount += tokens.get("reportMouseDown") == null ? 0 : tokens.get("reportMouseDown");
        sensingAmount += tokens.get("reportIsFastTracking") == null ? 0 : tokens.get("reportIsFastTracking");
        if(sensingAmount >= 2 || receiveMsgOrCloneAmount >= 2) {
            return 3;
        } else if(spritePressedAmount >= 2) {
            return 2;
        } else if(greenFlagAmount >= 2) {
            return 1;
        }
        return 0;
    }
    
    private Integer assesSynchronization(Map<String, Integer> tokens) {
        Boolean doWait = tokens.containsKey("doWait");
        Boolean doSayFor = tokens.containsKey("doSayFor");
        Boolean doThinkFor = tokens.containsKey("doThinkFor");
        Boolean doBroadcastAndWait = tokens.containsKey("doBroadcastAndWait");
        Boolean doWaitUntil = tokens.containsKey("doWaitUntil");
        if(doWaitUntil) {
            return 3;
        } else if(doSayFor || doThinkFor || doBroadcastAndWait) {
            return 2;
        } else if(doWait) {
            return 1;
        }
        return 0;
    }
    
    private Integer assessUserInteractivity(Map<String, Integer> tokens) {
        Boolean doPlayNote = tokens.containsKey("doPlayNote");
        Boolean doPlaySoundUntilDone = tokens.containsKey("doPlaySoundUntilDone");
        Boolean playSound = tokens.containsKey("playSound");
        Boolean receiveKey = tokens.containsKey("receiveKey");
        Boolean receiveInteraction = tokens.containsKey("receiveInteraction");
        Boolean reportMouseDown = tokens.containsKey("reportMouseDown");
        Boolean doAsk = tokens.containsKey("doAsk");
        Boolean reportTouchingColor = tokens.containsKey("reportTouchingColor");
        Boolean reportTouchingObject = tokens.containsKey("reportTouchingObject");
        Boolean bubble = tokens.containsKey("bubble");
        Boolean doSayFor = tokens.containsKey("doSayFor");
        Boolean doThinkFor = tokens.containsKey("doThinkFor");
        Boolean receiveGo = tokens.containsKey("receiveGo");
        if(doPlayNote || doPlaySoundUntilDone || playSound) {
            return 3;
        } else if (receiveKey || receiveInteraction || reportMouseDown || doAsk || reportTouchingColor || reportTouchingObject || bubble || doSayFor || doThinkFor) {
            return 2;
        } else if (receiveGo) {
            return 1;
        }
        return 0;
    }
    
    private Integer assessAbstraction(Map<String, Integer> tokens) {
        Integer scriptAmount = tokens.get("script") == null ? 0 : tokens.get("script");
        Boolean customBlock = tokens.containsKey("custom-block");
        Boolean receiveOnClone = tokens.containsKey("receiveOnClone");
        Boolean createClone = tokens.containsKey("createClone");
        if(receiveOnClone || createClone) {
            return 3;
        } else if (customBlock) {
            return 2;
        } else if(scriptAmount >= 2) {
            return 1;
        }
        return 0;
    }
    
    private Integer assessDataRepresentation(Map<String, Integer> tokens) {
        Boolean reportNewList = tokens.containsKey("reportNewList");
        Boolean var = tokens.containsKey("var=");
        if(reportNewList) {
            return 3;
        } else if(var) {
            return 2;
        } else {
            Boolean forward = tokens.containsKey("forward");
            Boolean gotoXY = tokens.containsKey("gotoXY");
            Boolean doGlide = tokens.containsKey("doGlide");
            Boolean setXPosition = tokens.containsKey("setXPosition");
            Boolean setYPosition = tokens.containsKey("setYPosition");
            Boolean changeXPosition = tokens.containsKey("changeXPosition");
            Boolean changeYPosition = tokens.containsKey("changeYPosition");
            Boolean changeScale = tokens.containsKey("changeScale");
            Boolean setScale = tokens.containsKey("setScale");
            Boolean setHeading = tokens.containsKey("setHeading");
            Boolean doFaceTowards = tokens.containsKey("doFaceTowards");
            Boolean turn = tokens.containsKey("turn");
            Boolean turnLeft = tokens.containsKey("turnLeft");
            Boolean doSwitchToCostume = tokens.containsKey("doSwitchToCostume");
            Boolean doWearNextCostume = tokens.containsKey("doWearNextCostume");
            Boolean changeEffect = tokens.containsKey("changeEffect");
            Boolean setEffect = tokens.containsKey("setEffect");
            Boolean show = tokens.containsKey("show");
            Boolean hide = tokens.containsKey("hide");
            if(forward || gotoXY || doGlide || setYPosition || setXPosition || changeXPosition || changeYPosition || changeScale || setScale || setHeading || doFaceTowards || turn || turnLeft || doSwitchToCostume || doWearNextCostume || changeEffect || setEffect || show || hide) {
                return 1;
            }
        }
        return 0;
    }
    
    private Integer assessFlowControl(Map<String, Integer> tokens) {
        Integer blockAmount = tokens.get("block") == null ? 0 : tokens.get("block");
        Boolean doRepeat = tokens.containsKey("doRepeat");
        Boolean doForever = tokens.containsKey("doForever");
        Boolean doUntil = tokens.containsKey("doUntil");
        if(doUntil) {
            return 3;
        } else if(doRepeat || doForever) {
            return 2;
        } else if(blockAmount >= 5) {
            return 1;
        }
        return 0;
    }
    
    private Integer assessOperators(Map<String, Integer> tokens) {
        Boolean usesComparisonsOperators =  tokens.containsKey("reportEquals") || 
                                            tokens.containsKey("reportLessThan") || 
                                            tokens.containsKey("reportGreaterThan");
        Boolean usesBasicMathOperators =    tokens.containsKey("reportSum") || 
                                            tokens.containsKey("reportDifference") || 
                                            tokens.containsKey("reportProduct") || 
                                            tokens.containsKey("reportQuotient");
        Boolean usesAdvancedMathOperators = tokens.containsKey("reportModulus") || 
                                            tokens.containsKey("reportRound") || 
                                            tokens.containsKey("reportMonadic") || 
                                            tokens.containsKey("reportRandom");
        Boolean usesStringOperators =       tokens.containsKey("reportJoinWords") || 
                                            tokens.containsKey("reportTextSplit") || 
                                            tokens.containsKey("reportLetter") || 
                                            tokens.containsKey("reportStringSize") ||
                                            tokens.containsKey("reportIsIdentical") || 
                                            tokens.containsKey("reportUnicode") || 
                                            tokens.containsKey("reportUnicodeAsLetter");
        ArrayList<Boolean> categoriesUsed = new ArrayList<>();
        categoriesUsed.add(usesComparisonsOperators);
        categoriesUsed.add(usesBasicMathOperators);
        categoriesUsed.add(usesAdvancedMathOperators);
        categoriesUsed.add(usesStringOperators);
        Integer count = 0;
        for(Boolean used : categoriesUsed)
            if(used)
                count++;
        if(count > 2) {
            return 3;
        } else {
            return count;
        }         
    }
    
    public Integer assessTotal(SnapGrade grade) {
        Integer total = 0;
        total += grade.getLogic();
        total += grade.getParallelism();
        total += grade.getSynchronization();
        total += grade.getUserInteractivity();
        total += grade.getAbstraction();
        total += grade.getDataRepresentation();
        total += grade.getFlowControl();
        total += grade.getOperators();
        return total;
    }
    
    public Double asessScore(SnapGrade grade, ProjectSettings settings) {
        ArrayList<Boolean> listOfSnapCriteria = new ArrayList<>();
        listOfSnapCriteria.add(settings.getLogicIsRelevant());
        listOfSnapCriteria.add(settings.getParallelismIsRelevant());
        listOfSnapCriteria.add(settings.getSynchronizationIsRelevant());
        listOfSnapCriteria.add(settings.getUserInteractivityIsRelevant());
        listOfSnapCriteria.add(settings.getAbstractionIsRelevant());
        listOfSnapCriteria.add(settings.getDataRepresentationIsRelevant());
        listOfSnapCriteria.add(settings.getFlowControlIsRelevant());
        listOfSnapCriteria.add(settings.getSnapOperatorsIsRelevant());
        Integer counter = 0;
        for(Boolean criteria : listOfSnapCriteria)
            if(criteria)
                counter++;
        if(counter == 0)
            counter = 1;
        return (grade.getTotal()/(counter*3.0))*10.0;
    }
    
    
    public String assessProjectLevel(SnapGrade grade) {
        Double score = grade.getScore();
        String projectLevel = "";
        if(score < 1.0) {
            projectLevel = "faixa branca";
        } else if(score < 2.0) {
            projectLevel = "faixa amarela";
        } else if(score < 3.0) {
            projectLevel = "faixa laranja";
        } else if(score < 4.0) {
            projectLevel = "faixa vermelha";
        } else if(score < 5.0) {
            projectLevel = "faixa roxa";
        } else if(score < 6.0) {
            projectLevel = "faixa azul";
        } else if(score < 7.0) {
            projectLevel = "faixa turquesa";
        } else if(score < 8.0) {
            projectLevel = "faixa verde";
        } else if(score < 9.0) {
            projectLevel = "faixa marrom";
        } else if(score <= 10.0) {
            projectLevel = "faixa preta";
        }
        return projectLevel;
    }
}
