package br.ufsc.cne.codemaster.restgrader.snapgrader;

import java.io.File;
import java.util.ArrayList;
import java.util.Map;
import org.apache.log4j.Logger;

import br.ufsc.cne.codemaster.restgrader.computacaonaescola.restgrader.Grade;
import br.ufsc.cne.codemaster.restgrader.computacaonaescola.restgrader.Grader;
import br.ufsc.cne.codemaster.restgrader.computacaonaescola.restgrader.ProjectSettings;

public class SnapMaster extends Grader {
    
    private SnapParser parser;
    private SnapGrader grader;
    static Logger log = Logger.getLogger(SnapMaster.class);
    
    
    public SnapMaster() {
        parser = new SnapParser();
        grader = new SnapGrader();
    }

    
    @Override
    public Grade gradeProject(File project, String projectName, ProjectSettings settings) {
        log.info("Iniciando análise do projeto " + projectName);
        ArrayList<String> tokens = getTokens(project);
        Map<String, Integer> frequencyOfTokens = getFrequencyOfTokens(tokens);
        log.info("Análise pronta.");
        log.info("Iniciando avaliação do projeto " + projectName);
        SnapGrade grade = assessCode(frequencyOfTokens, projectName, settings);
        if(settings.getParallelismIsRelevant() && grade.getParallelism() < 2 && frequencyOfTokens.containsKey("receiveKey") && frequencyOfTokens.get("receiveKey") > 1) {
            ArrayList<Integer> indexOfReceiveKey = findReceiveKeys(tokens);
            ArrayList<String> keys = findKeys(indexOfReceiveKey, tokens);
            if(twoKeysAreEqual(keys)) {
                grade.setParallelism(2);
                grade.setTotal(grader.assessTotal(grade));
                grade.setScore(grader.asessScore(grade, settings));
                grade.setProjectLevel(grader.assessProjectLevel(grade));
            }
        }
        log.info("Avaliação pronta.");
        return grade;
    }
    
    private ArrayList<String> getTokens(File project) {
        return parser.getTokens(project);
    }

    private Map<String, Integer> getFrequencyOfTokens(ArrayList<String> tokens) {
        return parser.getFrequencyOfTokens(tokens);
    }

    private SnapGrade assessCode(Map<String, Integer> frequencyOfTokens, String projectName, ProjectSettings settings) {
        return grader.assessProject(frequencyOfTokens, projectName, settings);
    }

    private ArrayList<Integer> findReceiveKeys(ArrayList<String> tokens) {
        return parser.findIndexOfReceiveKey(tokens);
    }
    
    private ArrayList<String> findKeys(ArrayList<Integer> indexOfReceiveKey, ArrayList<String> tokens) {
        ArrayList<String> keys = new ArrayList<>();
        for(int i=0; i<indexOfReceiveKey.size(); i++) {
            int index = indexOfReceiveKey.get(i);
            keys.add(tokens.get(index+3));
        }
        return keys;
    }

    private boolean twoKeysAreEqual(ArrayList<String> keys) {
        Boolean twoAreEqual = false;
        for(int j=0; j<keys.size(); j++)
            for(int k=j+1; k<keys.size(); k++)
                if(k!=j && keys.get(k).equals(keys.get(j)))
                    twoAreEqual = true;
        return twoAreEqual;
    }
}
