package br.ufsc.cne.codemaster.restgrader.snapgrader;

import java.io.File;
import java.io.IOException;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.regex.Pattern;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

public class SnapParser {
    
    static Logger log = Logger.getLogger(SnapParser.class);
    
    public ArrayList<String> getTokens(File xml) {
        Document xmlDocument = this.xmlToDocument(xml);
        String xmlString = this.documentToString(xmlDocument);
        return this.stringToTokens(xmlString);
    }
	
    public HashMap<String, Integer> getFrequencyOfTokens(ArrayList<String> tokens) {
        return this.countTokens(tokens);
    }

    private Document xmlToDocument(File xml) {
        try {
            DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
            DocumentBuilder db = dbf.newDocumentBuilder();
            Document document = (Document) db.parse(xml);
            return document;
        } catch (IOException | ParserConfigurationException | SAXException ex) {
            log.error("Erro ao converter o arquivo XML para Document", ex);
            return null;
        }
    }
	
    private String documentToString(Document doc) {
        try {
            TransformerFactory tfactory = TransformerFactory.newInstance();
            Transformer transformer = tfactory.newTransformer();
            transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
            StringWriter writer = new StringWriter();
            transformer.transform(new DOMSource(doc), new StreamResult(writer));
            String output = writer.getBuffer().toString().replaceAll("\n|\r", "");
            return output;
	} catch (Exception ex) {
            log.error("Erro ao converter o arquivo XML para String", ex);
            return null;
	}
    }
	
    private ArrayList<String> stringToTokens(String s) {
    	String splitCharacters = "  <>\"";
        String splitter = "[" + Pattern.quote(splitCharacters) + "]";
	String[] arrayOfTokens = s.split(splitter);
        ArrayList<String> arrayList = new ArrayList<>();
        for(String token : arrayOfTokens) {
            if(token.length() > 0) {
                arrayList.add(token);
            }
        }
        return arrayList;
    }

    private HashMap<String, Integer> countTokens(ArrayList<String> tokens) {
        HashMap<String, Integer> mapOfTokens = new HashMap<>();
        for(String token : tokens) {
            this.putTokenInMap(token, mapOfTokens);
        }
	return mapOfTokens;
    }
    
    private void putTokenInMap(String token, HashMap<String, Integer> mapOfTokens) {
        if(mapOfTokens.containsKey(token)) {
                Integer previousAmount = mapOfTokens.get(token);
                mapOfTokens.put(token, previousAmount + 1);
        } else {
            mapOfTokens.put(token, 1);
        }
    }

    public ArrayList<Integer> findIndexOfReceiveKey(ArrayList<String> tokens) {
        ArrayList<Integer> indexOfReceiveKey = new ArrayList<>();
        String match = "receiveKey";
        for(int i=0; i<tokens.size(); i++) {
            if(match.equals(tokens.get(i))) {
                indexOfReceiveKey.add(i);
            }
        }
        return indexOfReceiveKey;
    }
}
